<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

defined('MOODLE_INTERNAL') || die();

class nonsubmitters_message {

    /**
     * Send non-submitters message to students.
     *
     * @param string $message
     * @return void
     */
    public function send_message($userid, $subject, $message, $courseid) {
        global $CFG;

        // Pre 2.9 does not have \core\message\message()
        if ($CFG->branch >= 29) {
            $eventdata = new \core\message\message();
        } else {
            $eventdata = new stdClass();
        }

        $eventdata->component         = 'mod_turnitintooltwo';
        $eventdata->name              = 'nonsubmitters'; // This is the message name from messages.php.
        $eventdata->userfrom          = \core_user::get_noreply_user();
        $eventdata->userto            = $userid;
        $eventdata->subject           = $subject;
        $eventdata->fullmessage       = $message;
        $eventdata->fullmessageformat = FORMAT_HTML;
        $eventdata->fullmessagehtml   = $message;
        $eventdata->smallmessage      = '';
        $eventdata->notification      = 1; // This is only set to 0 for personal messages between users.

        if ($CFG->branch >= 32) {
            $eventdata->courseid = $courseid;
        }

        message_send($eventdata);
    }
}
